import { a as COMMON_TAB_ID, P as PARAMETER_JSON_CONFIG, b as configDirPath, d as CALCULATION_CONFIG, e as configFilesFolderPath, J as JSON_CONFIG, f as HOME_CONFIG, h as homeInfoDirName, g as digitalFilterCalculationFilesFolderPath, i as codeConversionCalculationFilesFolderPath } from './p-3ab399ac.js';
import { f as forceUpdate } from './p-597600b0.js';

const validateFileData = function (filePath, fileData, selectedDevice) {
  const fileName = filePath.replace(/^.*[\\\/]/, "");
  switch (fileName) {
    case "crc-parameters.json":
      if (!fileData[selectedDevice]) {
        throw new Error("CRC Parameters for " + selectedDevice + " are missing");
      }
      validateCrcData(fileData[selectedDevice], selectedDevice);
      break;
    case "digital-filter-parameters.json":
      if (!fileData[selectedDevice]) {
        throw new Error("Digital Filter Parameters for " + selectedDevice + " are missing");
      }
      validateDftData(fileData[selectedDevice]);
      break;
    case `${selectedDevice.toLowerCase()}_digital-filter-calculation.js`:
      validateModuleData(fileData[selectedDevice].calculationModule, selectedDevice);
      break;
    case "tue-parameters.json":
      if (!fileData[selectedDevice]) {
        throw new Error("TUE Parameters for " + selectedDevice + " are missing");
      }
      validateTueData(fileData[selectedDevice], selectedDevice);
      break;
  }
};
const validateCrcData = function (fileData, selectedDevice) {
  const regexp = /^(((0x|\$)[a-f0-9]+)|([a-f0-9]+h))$/i;
  for (let i = 0; i < fileData.length; i++) {
    if (fileData[i].displayName === undefined) {
      throw new Error("Display Name parameter for " + selectedDevice + " is missing");
    }
    else if (fileData[i].displayName === "" ||
      typeof fileData[i].displayName !== "string") {
      throw new Error("Invalid Display Name for " + selectedDevice);
    }
    else if (fileData[i].crcBitWidth === undefined) {
      throw new Error("CRC Bit Width parameter for " + selectedDevice + " is missing");
    }
    else if (fileData[i].crcBitWidth <= 0 ||
      typeof fileData[i].crcBitWidth !== "number") {
      throw new Error("Invalid CRC Bit Width for " + selectedDevice);
    }
    else if (fileData[i].seedValue === undefined) {
      throw new Error("Seed Value parameter for " + selectedDevice + " is missing");
    }
    else if (fileData[i].seedValue === "" ||
      typeof fileData[i].seedValue !== "string" ||
      !regexp.test(fileData[i].seedValue)) {
      throw new Error("Invalid Seed Value for " + selectedDevice);
    }
    else if (fileData[i].polynomial === undefined) {
      throw new Error("Polynomial parameter for " + selectedDevice + " is missing");
    }
    else if (fileData[i].polynomial === "" ||
      typeof fileData[i].polynomial !== "string" ||
      !regexp.test(fileData[i].polynomial)) {
      throw new Error("Invalid Polynomial for " + selectedDevice);
    }
  }
};
const validateDftData = function (fileData, selectedDevice) {
  const fileParameters = Object.keys(fileData);
  const validParameters = [
    "lutFileURL",
    "filterType",
    "dataRate",
    "programmableDelay",
    "globalChop",
    "clockFrequency",
  ];
  const invalidParameters = fileParameters.filter(parameter => !validParameters.includes(parameter));
  if (invalidParameters.length > 0) {
    throw new Error(`Invalid Parameters found in the Digital Filter Parameter file: ${invalidParameters.join(", ")}`);
  }
};
const validateModuleData = function (module, selectedDevice) {
  if (typeof module.deviceConfig === "undefined") {
    throw new Error("deviceConfig for " + selectedDevice + " is missing");
  }
  else if (typeof module.getFilterResponse !== "function") {
    throw new Error("getFilterResponse function for " + selectedDevice + " is missing");
  }
  else if (typeof module.getFilterSettlingTime !== "function") {
    throw new Error("getFilterSettlingTime function for " + selectedDevice + " is missing");
  }
};
const validateTueData = function (fileData, selectedDevice) {
  if (fileData.vrefVoltage === undefined) {
    throw new Error("VREF Voltage parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.vrefVoltage !== "string" ||
    fileData.vrefVoltage === "") {
    throw new Error("Invalid VREF Voltage for " + selectedDevice);
  }
  else if (fileData.internalGain === undefined) {
    throw new Error("Internal Gain parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.internalGain !== "string" ||
    fileData.internalGain === "") {
    throw new Error("Invalid Internal Gain for " + selectedDevice);
  }
  else if (fileData.minimumAmbientTemperature === undefined) {
    throw new Error("Minimum Ambient Temperature parameter for " +
      selectedDevice +
      " is missing");
  }
  else if (typeof fileData.minimumAmbientTemperature !== "string" ||
    fileData.minimumAmbientTemperature === "" ||
    parseFloat(fileData.minimumAmbientTemperature) >=
      parseFloat(fileData.maximumAmbientTemperature)) {
    throw new Error("Invalid Minimum Ambient Temperature for " + selectedDevice);
  }
  else if (fileData.maximumAmbientTemperature === undefined) {
    throw new Error("Maximum Ambient Temperature parameter for " +
      selectedDevice +
      " is missing");
  }
  else if (typeof fileData.maximumAmbientTemperature !== "string" ||
    fileData.maximumAmbientTemperature === "" ||
    parseFloat(fileData.maximumAmbientTemperature) <=
      parseFloat(fileData.minimumAmbientTemperature)) {
    throw new Error("Invalid Maximum Ambient Temperature for " + selectedDevice);
  }
  else if (fileData.adcScalingFactor === undefined) {
    throw new Error("ADC Scaling Factor parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcScalingFactor !== "string" ||
    fileData.adcScalingFactor === "") {
    throw new Error("Invalid ADC Scaling Factor for " + selectedDevice);
  }
  else if (fileData.minimumAdcVrefRange === undefined) {
    throw new Error("Minimum ADC VREF Range parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.minimumAdcVrefRange !== "string" ||
    fileData.minimumAdcVrefRange === "" ||
    parseFloat(fileData.minimumAdcVrefRange) >
      parseFloat(fileData.maximumAdcVrefRange)) {
    throw new Error("Invalid Minimum ADC VREF Range for " + selectedDevice);
  }
  else if (fileData.maximumAdcVrefRange === undefined) {
    throw new Error("Maximum ADC VREF Range parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.maximumAdcVrefRange !== "string" ||
    fileData.maximumAdcVrefRange === "" ||
    parseFloat(fileData.maximumAdcVrefRange) <
      parseFloat(fileData.minimumAdcVrefRange)) {
    throw new Error("Invalid Maximum ADC VREF Range for " + selectedDevice);
  }
  else if (fileData.minimumAdcTempRange === undefined) {
    throw new Error("Minimum ADC Temperature Range parameter for " +
      selectedDevice +
      " is missing");
  }
  else if (typeof fileData.minimumAdcTempRange !== "string" ||
    fileData.minimumAdcTempRange === "" ||
    parseFloat(fileData.minimumAdcTempRange) >=
      parseFloat(fileData.maximumAdcTempRange)) {
    throw new Error("Invalid Minimum ADC Temperature Range for " + selectedDevice);
  }
  else if (fileData.maximumAdcTempRange === undefined) {
    throw new Error("Maximum ADC Temperature Range parameter for " +
      selectedDevice +
      " is missing");
  }
  else if (typeof fileData.maximumAdcTempRange !== "string" ||
    fileData.maximumAdcTempRange === "" ||
    parseFloat(fileData.maximumAdcTempRange) <=
      parseFloat(fileData.minimumAdcTempRange)) {
    throw new Error("Invalid Maximum ADC Temperature Range for " + selectedDevice);
  }
  else if (fileData.resolution === undefined) {
    throw new Error("Resolution parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.resolution !== "string" ||
    fileData.resolution === "") {
    throw new Error("Invalid Resolution for " + selectedDevice);
  }
  else if (fileData.adcNoise === undefined) {
    throw new Error("ADC Noise parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcNoise !== "object") {
    throw new Error("Invalid ADC Noise for " + selectedDevice);
  }
  else if (fileData.adcNoise.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC Noise is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcNoise.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC Noise in " + selectedDevice);
  }
  else if (fileData.adcNoise.value === undefined) {
    throw new Error("The Value parameter for ADC Noise is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcNoise.value !== "string") {
    throw new Error("Invalid Value for ADC Noise in " + selectedDevice);
  }
  else if (fileData.adcNoise.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC Noise is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcNoise.unitOptions !== "string" ||
    fileData.adcNoise.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC Noise in " + selectedDevice);
  }
  else if (fileData.adcNoise.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC Noise is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcNoise.defaultUnit !== "string" ||
    fileData.adcNoise.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC Noise in " + selectedDevice);
  }
  else if (fileData.adcOffset === undefined) {
    throw new Error("ADC Offset parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcOffset !== "object") {
    throw new Error("Invalid ADC Offset for " + selectedDevice);
  }
  else if (fileData.adcOffset.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC Offset is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcOffset.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC Offset in " + selectedDevice);
  }
  else if (fileData.adcOffset.value === undefined) {
    throw new Error("The Value parameter for ADC Offset is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcOffset.value !== "string") {
    throw new Error("Invalid Value for ADC Offset in " + selectedDevice);
  }
  else if (fileData.adcOffset.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC Offset is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffset.unitOptions !== "string" ||
    fileData.adcOffset.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC Offset in " + selectedDevice);
  }
  else if (fileData.adcOffset.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC Offset is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffset.defaultUnit !== "string" ||
    fileData.adcOffset.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC Offset in " + selectedDevice);
  }
  else if (fileData.adcGainError === undefined) {
    throw new Error("ADC Gain Error parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcGainError !== "object") {
    throw new Error("Invalid ADC Gain Error for " + selectedDevice);
  }
  else if (fileData.adcGainError.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC Gain Error is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainError.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC Gain Error in " + selectedDevice);
  }
  else if (fileData.adcGainError.value === undefined) {
    throw new Error("The Value parameter for ADC Gain Error is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcGainError.value !== "string") {
    throw new Error("Invalid Value for ADC Gain Error in " + selectedDevice);
  }
  else if (fileData.adcGainError.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC Gain Error is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainError.unitOptions !== "string" ||
    fileData.adcGainError.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC Gain Error in " + selectedDevice);
  }
  else if (fileData.adcGainError.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC Gain Error is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainError.defaultUnit !== "string" ||
    fileData.adcGainError.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC Gain Error in " + selectedDevice);
  }
  else if (fileData.adcInl === undefined) {
    throw new Error("ADC INL parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcInl !== "object") {
    throw new Error("Invalid ADC INL for " + selectedDevice);
  }
  else if (fileData.adcInl.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC INL is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcInl.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC INL in " + selectedDevice);
  }
  else if (fileData.adcInl.value === undefined) {
    throw new Error("The Value parameter for ADC INL is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcInl.value !== "string") {
    throw new Error("Invalid Value for ADC INL in " + selectedDevice);
  }
  else if (fileData.adcInl.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC INL is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcInl.unitOptions !== "string" ||
    fileData.adcInl.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC INL in " + selectedDevice);
  }
  else if (fileData.adcInl.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC INL is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcInl.defaultUnit !== "string" ||
    fileData.adcInl.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC INL in " + selectedDevice);
  }
  else if (fileData.adcOffsetDrift === undefined) {
    throw new Error("ADC Offset Drift parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcOffsetDrift !== "object") {
    throw new Error("Invalid ADC Offset Drift for " + selectedDevice);
  }
  else if (fileData.adcOffsetDrift.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC Offset Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffsetDrift.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC Offset Drift in " + selectedDevice);
  }
  else if (fileData.adcOffsetDrift.value === undefined) {
    throw new Error("The Value parameter for ADC Offset Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffsetDrift.value !== "string") {
    throw new Error("Invalid Value for ADC Offset Drift in " + selectedDevice);
  }
  else if (fileData.adcOffsetDrift.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC Offset Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffsetDrift.unitOptions !== "string" ||
    fileData.adcOffsetDrift.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC Offset Drift in " + selectedDevice);
  }
  else if (fileData.adcOffsetDrift.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC Offset Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcOffsetDrift.defaultUnit !== "string" ||
    fileData.adcOffsetDrift.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC Offset Drift in " + selectedDevice);
  }
  else if (fileData.adcGainDrift === undefined) {
    throw new Error("ADC Gain Drift parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.adcGainDrift !== "object") {
    throw new Error("Invalid ADC Gain Drift for " + selectedDevice);
  }
  else if (fileData.adcGainDrift.tooltip === undefined) {
    throw new Error("The Tooltip parameter for ADC Gain Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainDrift.tooltip !== "string") {
    throw new Error("Invalid Tooltip for ADC Gain Drift in " + selectedDevice);
  }
  else if (fileData.adcGainDrift.value === undefined) {
    throw new Error("The Value parameter for ADC Gain Drift is missing in " + selectedDevice);
  }
  else if (typeof fileData.adcGainDrift.value !== "string") {
    throw new Error("Invalid Value for ADC Gain Drift in " + selectedDevice);
  }
  else if (fileData.adcGainDrift.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for ADC Gain Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainDrift.unitOptions !== "string" ||
    fileData.adcGainDrift.unitOptions === "") {
    throw new Error("Invalid Unit Options for ADC Gain Drift in " + selectedDevice);
  }
  else if (fileData.adcGainDrift.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for ADC Gain Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.adcGainDrift.defaultUnit !== "string" ||
    fileData.adcGainDrift.defaultUnit === "") {
    throw new Error("Invalid Default Unit for ADC Gain Drift in " + selectedDevice);
  }
  else if (fileData.vrefNoise === undefined) {
    throw new Error("VREF Noise parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.vrefNoise !== "object") {
    throw new Error("Invalid VREF Noise for " + selectedDevice);
  }
  else if (fileData.vrefNoise.value === undefined) {
    throw new Error("The Value parameter for VREF Noise is missing in " + selectedDevice);
  }
  else if (typeof fileData.vrefNoise.value !== "string") {
    throw new Error("Invalid Value for VREF Noise in " + selectedDevice);
  }
  else if (fileData.vrefNoise.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for VREF Noise is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefNoise.unitOptions !== "string" ||
    fileData.vrefNoise.unitOptions === "") {
    throw new Error("Invalid Unit Options for VREF Noise in " + selectedDevice);
  }
  else if (fileData.vrefNoise.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for VREF Noise is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefNoise.defaultUnit !== "string" ||
    fileData.vrefNoise.defaultUnit === "") {
    throw new Error("Invalid Default Unit for VREF Noise in " + selectedDevice);
  }
  else if (fileData.vrefInitialAccuracy === undefined) {
    throw new Error("VREF Initial Accuracy parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.vrefInitialAccuracy !== "object") {
    throw new Error("Invalid VREF Initial Accuracy for " + selectedDevice);
  }
  else if (fileData.vrefInitialAccuracy.value === undefined) {
    throw new Error("The Value parameter for VREF Initial Accuracy is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefInitialAccuracy.value !== "string") {
    throw new Error("Invalid Value for VREF Initial Accuracy in " + selectedDevice);
  }
  else if (fileData.vrefInitialAccuracy.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for VREF Initial Accuracy is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefInitialAccuracy.unitOptions !== "string" ||
    fileData.vrefInitialAccuracy.unitOptions === "") {
    throw new Error("Invalid Unit Options for VREF Initial Accuracy in " + selectedDevice);
  }
  else if (fileData.vrefInitialAccuracy.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for VREF Initial Accuracy is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefInitialAccuracy.defaultUnit !== "string" ||
    fileData.vrefInitialAccuracy.defaultUnit === "") {
    throw new Error("Invalid Default Unit for VREF Initial Accuracy in " + selectedDevice);
  }
  else if (fileData.vrefTemperatureDrift === undefined) {
    throw new Error("VREF Temperature Drift parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.vrefTemperatureDrift !== "object") {
    throw new Error("Invalid VREF Temperature Drift for " + selectedDevice);
  }
  else if (fileData.vrefTemperatureDrift.value === undefined) {
    throw new Error("The Value parameter for VREF Temperature Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefTemperatureDrift.value !== "string") {
    throw new Error("Invalid Value for VREF Temperature Drift in " + selectedDevice);
  }
  else if (fileData.vrefTemperatureDrift.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for VREF Temperature Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefTemperatureDrift.unitOptions !== "string" ||
    fileData.vrefTemperatureDrift.unitOptions === "") {
    throw new Error("Invalid Unit Options for VREF Temperature Drift in " + selectedDevice);
  }
  else if (fileData.vrefTemperatureDrift.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for VREF Temperature Drift is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.vrefTemperatureDrift.defaultUnit !== "string" ||
    fileData.vrefTemperatureDrift.defaultUnit === "") {
    throw new Error("Invalid Default Unit for VREF Temperature Drift in " + selectedDevice);
  }
  else if (fileData.outputUnit === undefined) {
    throw new Error("Output Unit parameter for " + selectedDevice + " is missing");
  }
  else if (typeof fileData.outputUnit !== "object") {
    throw new Error("Invalid Output Unit for " + selectedDevice);
  }
  else if (fileData.outputUnit.unitOptions === undefined) {
    throw new Error("The Unit Options parameter for Output Unit is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.outputUnit.unitOptions !== "string" ||
    fileData.outputUnit.unitOptions === "") {
    throw new Error("Invalid Unit Options for Output Unit in " + selectedDevice);
  }
  else if (fileData.outputUnit.defaultUnit === undefined) {
    throw new Error("The Default Unit parameter for Output Unit is missing in " +
      selectedDevice);
  }
  else if (typeof fileData.outputUnit.defaultUnit !== "string" ||
    fileData.outputUnit.defaultUnit === "") {
    throw new Error("Invalid Default Unit for Output Unit in " + selectedDevice);
  }
};

/** Detect free variable `global` from Node.js. */
var freeGlobal = typeof global == 'object' && global && global.Object === Object && global;

/** Detect free variable `self`. */
var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

/** Used as a reference to the global object. */
var root = freeGlobal || freeSelf || Function('return this')();

/** Built-in value references. */
var Symbol = root.Symbol;

/** Used for built-in method references. */
var objectProto$5 = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty$4 = objectProto$5.hasOwnProperty;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var nativeObjectToString$1 = objectProto$5.toString;

/** Built-in value references. */
var symToStringTag$1 = Symbol ? Symbol.toStringTag : undefined;

/**
 * A specialized version of `baseGetTag` which ignores `Symbol.toStringTag` values.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the raw `toStringTag`.
 */
function getRawTag(value) {
  var isOwn = hasOwnProperty$4.call(value, symToStringTag$1),
      tag = value[symToStringTag$1];

  try {
    value[symToStringTag$1] = undefined;
    var unmasked = true;
  } catch (e) {}

  var result = nativeObjectToString$1.call(value);
  if (unmasked) {
    if (isOwn) {
      value[symToStringTag$1] = tag;
    } else {
      delete value[symToStringTag$1];
    }
  }
  return result;
}

/** Used for built-in method references. */
var objectProto$4 = Object.prototype;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var nativeObjectToString = objectProto$4.toString;

/**
 * Converts `value` to a string using `Object.prototype.toString`.
 *
 * @private
 * @param {*} value The value to convert.
 * @returns {string} Returns the converted string.
 */
function objectToString(value) {
  return nativeObjectToString.call(value);
}

/** `Object#toString` result references. */
var nullTag = '[object Null]',
    undefinedTag = '[object Undefined]';

/** Built-in value references. */
var symToStringTag = Symbol ? Symbol.toStringTag : undefined;

/**
 * The base implementation of `getTag` without fallbacks for buggy environments.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the `toStringTag`.
 */
function baseGetTag(value) {
  if (value == null) {
    return value === undefined ? undefinedTag : nullTag;
  }
  return (symToStringTag && symToStringTag in Object(value))
    ? getRawTag(value)
    : objectToString(value);
}

/**
 * Checks if `value` is object-like. A value is object-like if it's not `null`
 * and has a `typeof` result of "object".
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
 * @example
 *
 * _.isObjectLike({});
 * // => true
 *
 * _.isObjectLike([1, 2, 3]);
 * // => true
 *
 * _.isObjectLike(_.noop);
 * // => false
 *
 * _.isObjectLike(null);
 * // => false
 */
function isObjectLike(value) {
  return value != null && typeof value == 'object';
}

/** `Object#toString` result references. */
var symbolTag = '[object Symbol]';

/**
 * Checks if `value` is classified as a `Symbol` primitive or object.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
 * @example
 *
 * _.isSymbol(Symbol.iterator);
 * // => true
 *
 * _.isSymbol('abc');
 * // => false
 */
function isSymbol(value) {
  return typeof value == 'symbol' ||
    (isObjectLike(value) && baseGetTag(value) == symbolTag);
}

/**
 * A specialized version of `_.map` for arrays without support for iteratee
 * shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the new mapped array.
 */
function arrayMap(array, iteratee) {
  var index = -1,
      length = array == null ? 0 : array.length,
      result = Array(length);

  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}

/**
 * Checks if `value` is classified as an `Array` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array, else `false`.
 * @example
 *
 * _.isArray([1, 2, 3]);
 * // => true
 *
 * _.isArray(document.body.children);
 * // => false
 *
 * _.isArray('abc');
 * // => false
 *
 * _.isArray(_.noop);
 * // => false
 */
var isArray = Array.isArray;

/** Used as references for various `Number` constants. */
var INFINITY$1 = 1 / 0;

/** Used to convert symbols to primitives and strings. */
var symbolProto = Symbol ? Symbol.prototype : undefined,
    symbolToString = symbolProto ? symbolProto.toString : undefined;

/**
 * The base implementation of `_.toString` which doesn't convert nullish
 * values to empty strings.
 *
 * @private
 * @param {*} value The value to process.
 * @returns {string} Returns the string.
 */
function baseToString(value) {
  // Exit early for strings to avoid a performance hit in some environments.
  if (typeof value == 'string') {
    return value;
  }
  if (isArray(value)) {
    // Recursively convert values (susceptible to call stack limits).
    return arrayMap(value, baseToString) + '';
  }
  if (isSymbol(value)) {
    return symbolToString ? symbolToString.call(value) : '';
  }
  var result = (value + '');
  return (result == '0' && (1 / value) == -INFINITY$1) ? '-0' : result;
}

/**
 * Checks if `value` is the
 * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
 * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an object, else `false`.
 * @example
 *
 * _.isObject({});
 * // => true
 *
 * _.isObject([1, 2, 3]);
 * // => true
 *
 * _.isObject(_.noop);
 * // => true
 *
 * _.isObject(null);
 * // => false
 */
function isObject(value) {
  var type = typeof value;
  return value != null && (type == 'object' || type == 'function');
}

/** `Object#toString` result references. */
var asyncTag = '[object AsyncFunction]',
    funcTag = '[object Function]',
    genTag = '[object GeneratorFunction]',
    proxyTag = '[object Proxy]';

/**
 * Checks if `value` is classified as a `Function` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a function, else `false`.
 * @example
 *
 * _.isFunction(_);
 * // => true
 *
 * _.isFunction(/abc/);
 * // => false
 */
function isFunction(value) {
  if (!isObject(value)) {
    return false;
  }
  // The use of `Object#toString` avoids issues with the `typeof` operator
  // in Safari 9 which returns 'object' for typed arrays and other constructors.
  var tag = baseGetTag(value);
  return tag == funcTag || tag == genTag || tag == asyncTag || tag == proxyTag;
}

/** Used to detect overreaching core-js shims. */
var coreJsData = root['__core-js_shared__'];

/** Used to detect methods masquerading as native. */
var maskSrcKey = (function() {
  var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
  return uid ? ('Symbol(src)_1.' + uid) : '';
}());

/**
 * Checks if `func` has its source masked.
 *
 * @private
 * @param {Function} func The function to check.
 * @returns {boolean} Returns `true` if `func` is masked, else `false`.
 */
function isMasked(func) {
  return !!maskSrcKey && (maskSrcKey in func);
}

/** Used for built-in method references. */
var funcProto$1 = Function.prototype;

/** Used to resolve the decompiled source of functions. */
var funcToString$1 = funcProto$1.toString;

/**
 * Converts `func` to its source code.
 *
 * @private
 * @param {Function} func The function to convert.
 * @returns {string} Returns the source code.
 */
function toSource(func) {
  if (func != null) {
    try {
      return funcToString$1.call(func);
    } catch (e) {}
    try {
      return (func + '');
    } catch (e) {}
  }
  return '';
}

/**
 * Used to match `RegExp`
 * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
 */
var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

/** Used to detect host constructors (Safari). */
var reIsHostCtor = /^\[object .+?Constructor\]$/;

/** Used for built-in method references. */
var funcProto = Function.prototype,
    objectProto$3 = Object.prototype;

/** Used to resolve the decompiled source of functions. */
var funcToString = funcProto.toString;

/** Used to check objects for own properties. */
var hasOwnProperty$3 = objectProto$3.hasOwnProperty;

/** Used to detect if a method is native. */
var reIsNative = RegExp('^' +
  funcToString.call(hasOwnProperty$3).replace(reRegExpChar, '\\$&')
  .replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$'
);

/**
 * The base implementation of `_.isNative` without bad shim checks.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a native function,
 *  else `false`.
 */
function baseIsNative(value) {
  if (!isObject(value) || isMasked(value)) {
    return false;
  }
  var pattern = isFunction(value) ? reIsNative : reIsHostCtor;
  return pattern.test(toSource(value));
}

/**
 * Gets the value at `key` of `object`.
 *
 * @private
 * @param {Object} [object] The object to query.
 * @param {string} key The key of the property to get.
 * @returns {*} Returns the property value.
 */
function getValue(object, key) {
  return object == null ? undefined : object[key];
}

/**
 * Gets the native function at `key` of `object`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {string} key The key of the method to get.
 * @returns {*} Returns the function if it's native, else `undefined`.
 */
function getNative(object, key) {
  var value = getValue(object, key);
  return baseIsNative(value) ? value : undefined;
}

var defineProperty = (function() {
  try {
    var func = getNative(Object, 'defineProperty');
    func({}, '', {});
    return func;
  } catch (e) {}
}());

/** Used as references for various `Number` constants. */
var MAX_SAFE_INTEGER = 9007199254740991;

/** Used to detect unsigned integer values. */
var reIsUint = /^(?:0|[1-9]\d*)$/;

/**
 * Checks if `value` is a valid array-like index.
 *
 * @private
 * @param {*} value The value to check.
 * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
 * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
 */
function isIndex(value, length) {
  var type = typeof value;
  length = length == null ? MAX_SAFE_INTEGER : length;

  return !!length &&
    (type == 'number' ||
      (type != 'symbol' && reIsUint.test(value))) &&
        (value > -1 && value % 1 == 0 && value < length);
}

/**
 * The base implementation of `assignValue` and `assignMergeValue` without
 * value checks.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {string} key The key of the property to assign.
 * @param {*} value The value to assign.
 */
function baseAssignValue(object, key, value) {
  if (key == '__proto__' && defineProperty) {
    defineProperty(object, key, {
      'configurable': true,
      'enumerable': true,
      'value': value,
      'writable': true
    });
  } else {
    object[key] = value;
  }
}

/**
 * Performs a
 * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
 * comparison between two values to determine if they are equivalent.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to compare.
 * @param {*} other The other value to compare.
 * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
 * @example
 *
 * var object = { 'a': 1 };
 * var other = { 'a': 1 };
 *
 * _.eq(object, object);
 * // => true
 *
 * _.eq(object, other);
 * // => false
 *
 * _.eq('a', 'a');
 * // => true
 *
 * _.eq('a', Object('a'));
 * // => false
 *
 * _.eq(NaN, NaN);
 * // => true
 */
function eq(value, other) {
  return value === other || (value !== value && other !== other);
}

/** Used for built-in method references. */
var objectProto$2 = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty$2 = objectProto$2.hasOwnProperty;

/**
 * Assigns `value` to `key` of `object` if the existing value is not equivalent
 * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
 * for equality comparisons.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {string} key The key of the property to assign.
 * @param {*} value The value to assign.
 */
function assignValue(object, key, value) {
  var objValue = object[key];
  if (!(hasOwnProperty$2.call(object, key) && eq(objValue, value)) ||
      (value === undefined && !(key in object))) {
    baseAssignValue(object, key, value);
  }
}

/** Used to match property names within property paths. */
var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
    reIsPlainProp = /^\w*$/;

/**
 * Checks if `value` is a property name and not a property path.
 *
 * @private
 * @param {*} value The value to check.
 * @param {Object} [object] The object to query keys on.
 * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
 */
function isKey(value, object) {
  if (isArray(value)) {
    return false;
  }
  var type = typeof value;
  if (type == 'number' || type == 'symbol' || type == 'boolean' ||
      value == null || isSymbol(value)) {
    return true;
  }
  return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
    (object != null && value in Object(object));
}

/* Built-in method references that are verified to be native. */
var nativeCreate = getNative(Object, 'create');

/**
 * Removes all key-value entries from the hash.
 *
 * @private
 * @name clear
 * @memberOf Hash
 */
function hashClear() {
  this.__data__ = nativeCreate ? nativeCreate(null) : {};
  this.size = 0;
}

/**
 * Removes `key` and its value from the hash.
 *
 * @private
 * @name delete
 * @memberOf Hash
 * @param {Object} hash The hash to modify.
 * @param {string} key The key of the value to remove.
 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
 */
function hashDelete(key) {
  var result = this.has(key) && delete this.__data__[key];
  this.size -= result ? 1 : 0;
  return result;
}

/** Used to stand-in for `undefined` hash values. */
var HASH_UNDEFINED$1 = '__lodash_hash_undefined__';

/** Used for built-in method references. */
var objectProto$1 = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty$1 = objectProto$1.hasOwnProperty;

/**
 * Gets the hash value for `key`.
 *
 * @private
 * @name get
 * @memberOf Hash
 * @param {string} key The key of the value to get.
 * @returns {*} Returns the entry value.
 */
function hashGet(key) {
  var data = this.__data__;
  if (nativeCreate) {
    var result = data[key];
    return result === HASH_UNDEFINED$1 ? undefined : result;
  }
  return hasOwnProperty$1.call(data, key) ? data[key] : undefined;
}

/** Used for built-in method references. */
var objectProto = Object.prototype;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto.hasOwnProperty;

/**
 * Checks if a hash value for `key` exists.
 *
 * @private
 * @name has
 * @memberOf Hash
 * @param {string} key The key of the entry to check.
 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
 */
function hashHas(key) {
  var data = this.__data__;
  return nativeCreate ? (data[key] !== undefined) : hasOwnProperty.call(data, key);
}

/** Used to stand-in for `undefined` hash values. */
var HASH_UNDEFINED = '__lodash_hash_undefined__';

/**
 * Sets the hash `key` to `value`.
 *
 * @private
 * @name set
 * @memberOf Hash
 * @param {string} key The key of the value to set.
 * @param {*} value The value to set.
 * @returns {Object} Returns the hash instance.
 */
function hashSet(key, value) {
  var data = this.__data__;
  this.size += this.has(key) ? 0 : 1;
  data[key] = (nativeCreate && value === undefined) ? HASH_UNDEFINED : value;
  return this;
}

/**
 * Creates a hash object.
 *
 * @private
 * @constructor
 * @param {Array} [entries] The key-value pairs to cache.
 */
function Hash(entries) {
  var index = -1,
      length = entries == null ? 0 : entries.length;

  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}

// Add methods to `Hash`.
Hash.prototype.clear = hashClear;
Hash.prototype['delete'] = hashDelete;
Hash.prototype.get = hashGet;
Hash.prototype.has = hashHas;
Hash.prototype.set = hashSet;

/**
 * Removes all key-value entries from the list cache.
 *
 * @private
 * @name clear
 * @memberOf ListCache
 */
function listCacheClear() {
  this.__data__ = [];
  this.size = 0;
}

/**
 * Gets the index at which the `key` is found in `array` of key-value pairs.
 *
 * @private
 * @param {Array} array The array to inspect.
 * @param {*} key The key to search for.
 * @returns {number} Returns the index of the matched value, else `-1`.
 */
function assocIndexOf(array, key) {
  var length = array.length;
  while (length--) {
    if (eq(array[length][0], key)) {
      return length;
    }
  }
  return -1;
}

/** Used for built-in method references. */
var arrayProto = Array.prototype;

/** Built-in value references. */
var splice = arrayProto.splice;

/**
 * Removes `key` and its value from the list cache.
 *
 * @private
 * @name delete
 * @memberOf ListCache
 * @param {string} key The key of the value to remove.
 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
 */
function listCacheDelete(key) {
  var data = this.__data__,
      index = assocIndexOf(data, key);

  if (index < 0) {
    return false;
  }
  var lastIndex = data.length - 1;
  if (index == lastIndex) {
    data.pop();
  } else {
    splice.call(data, index, 1);
  }
  --this.size;
  return true;
}

/**
 * Gets the list cache value for `key`.
 *
 * @private
 * @name get
 * @memberOf ListCache
 * @param {string} key The key of the value to get.
 * @returns {*} Returns the entry value.
 */
function listCacheGet(key) {
  var data = this.__data__,
      index = assocIndexOf(data, key);

  return index < 0 ? undefined : data[index][1];
}

/**
 * Checks if a list cache value for `key` exists.
 *
 * @private
 * @name has
 * @memberOf ListCache
 * @param {string} key The key of the entry to check.
 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
 */
function listCacheHas(key) {
  return assocIndexOf(this.__data__, key) > -1;
}

/**
 * Sets the list cache `key` to `value`.
 *
 * @private
 * @name set
 * @memberOf ListCache
 * @param {string} key The key of the value to set.
 * @param {*} value The value to set.
 * @returns {Object} Returns the list cache instance.
 */
function listCacheSet(key, value) {
  var data = this.__data__,
      index = assocIndexOf(data, key);

  if (index < 0) {
    ++this.size;
    data.push([key, value]);
  } else {
    data[index][1] = value;
  }
  return this;
}

/**
 * Creates an list cache object.
 *
 * @private
 * @constructor
 * @param {Array} [entries] The key-value pairs to cache.
 */
function ListCache(entries) {
  var index = -1,
      length = entries == null ? 0 : entries.length;

  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}

// Add methods to `ListCache`.
ListCache.prototype.clear = listCacheClear;
ListCache.prototype['delete'] = listCacheDelete;
ListCache.prototype.get = listCacheGet;
ListCache.prototype.has = listCacheHas;
ListCache.prototype.set = listCacheSet;

/* Built-in method references that are verified to be native. */
var Map = getNative(root, 'Map');

/**
 * Removes all key-value entries from the map.
 *
 * @private
 * @name clear
 * @memberOf MapCache
 */
function mapCacheClear() {
  this.size = 0;
  this.__data__ = {
    'hash': new Hash,
    'map': new (Map || ListCache),
    'string': new Hash
  };
}

/**
 * Checks if `value` is suitable for use as unique object key.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
 */
function isKeyable(value) {
  var type = typeof value;
  return (type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean')
    ? (value !== '__proto__')
    : (value === null);
}

/**
 * Gets the data for `map`.
 *
 * @private
 * @param {Object} map The map to query.
 * @param {string} key The reference key.
 * @returns {*} Returns the map data.
 */
function getMapData(map, key) {
  var data = map.__data__;
  return isKeyable(key)
    ? data[typeof key == 'string' ? 'string' : 'hash']
    : data.map;
}

/**
 * Removes `key` and its value from the map.
 *
 * @private
 * @name delete
 * @memberOf MapCache
 * @param {string} key The key of the value to remove.
 * @returns {boolean} Returns `true` if the entry was removed, else `false`.
 */
function mapCacheDelete(key) {
  var result = getMapData(this, key)['delete'](key);
  this.size -= result ? 1 : 0;
  return result;
}

/**
 * Gets the map value for `key`.
 *
 * @private
 * @name get
 * @memberOf MapCache
 * @param {string} key The key of the value to get.
 * @returns {*} Returns the entry value.
 */
function mapCacheGet(key) {
  return getMapData(this, key).get(key);
}

/**
 * Checks if a map value for `key` exists.
 *
 * @private
 * @name has
 * @memberOf MapCache
 * @param {string} key The key of the entry to check.
 * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
 */
function mapCacheHas(key) {
  return getMapData(this, key).has(key);
}

/**
 * Sets the map `key` to `value`.
 *
 * @private
 * @name set
 * @memberOf MapCache
 * @param {string} key The key of the value to set.
 * @param {*} value The value to set.
 * @returns {Object} Returns the map cache instance.
 */
function mapCacheSet(key, value) {
  var data = getMapData(this, key),
      size = data.size;

  data.set(key, value);
  this.size += data.size == size ? 0 : 1;
  return this;
}

/**
 * Creates a map cache object to store key-value pairs.
 *
 * @private
 * @constructor
 * @param {Array} [entries] The key-value pairs to cache.
 */
function MapCache(entries) {
  var index = -1,
      length = entries == null ? 0 : entries.length;

  this.clear();
  while (++index < length) {
    var entry = entries[index];
    this.set(entry[0], entry[1]);
  }
}

// Add methods to `MapCache`.
MapCache.prototype.clear = mapCacheClear;
MapCache.prototype['delete'] = mapCacheDelete;
MapCache.prototype.get = mapCacheGet;
MapCache.prototype.has = mapCacheHas;
MapCache.prototype.set = mapCacheSet;

/** Error message constants. */
var FUNC_ERROR_TEXT = 'Expected a function';

/**
 * Creates a function that memoizes the result of `func`. If `resolver` is
 * provided, it determines the cache key for storing the result based on the
 * arguments provided to the memoized function. By default, the first argument
 * provided to the memoized function is used as the map cache key. The `func`
 * is invoked with the `this` binding of the memoized function.
 *
 * **Note:** The cache is exposed as the `cache` property on the memoized
 * function. Its creation may be customized by replacing the `_.memoize.Cache`
 * constructor with one whose instances implement the
 * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
 * method interface of `clear`, `delete`, `get`, `has`, and `set`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Function
 * @param {Function} func The function to have its output memoized.
 * @param {Function} [resolver] The function to resolve the cache key.
 * @returns {Function} Returns the new memoized function.
 * @example
 *
 * var object = { 'a': 1, 'b': 2 };
 * var other = { 'c': 3, 'd': 4 };
 *
 * var values = _.memoize(_.values);
 * values(object);
 * // => [1, 2]
 *
 * values(other);
 * // => [3, 4]
 *
 * object.a = 2;
 * values(object);
 * // => [1, 2]
 *
 * // Modify the result cache.
 * values.cache.set(object, ['a', 'b']);
 * values(object);
 * // => ['a', 'b']
 *
 * // Replace `_.memoize.Cache`.
 * _.memoize.Cache = WeakMap;
 */
function memoize(func, resolver) {
  if (typeof func != 'function' || (resolver != null && typeof resolver != 'function')) {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  var memoized = function() {
    var args = arguments,
        key = resolver ? resolver.apply(this, args) : args[0],
        cache = memoized.cache;

    if (cache.has(key)) {
      return cache.get(key);
    }
    var result = func.apply(this, args);
    memoized.cache = cache.set(key, result) || cache;
    return result;
  };
  memoized.cache = new (memoize.Cache || MapCache);
  return memoized;
}

// Expose `MapCache`.
memoize.Cache = MapCache;

/** Used as the maximum memoize cache size. */
var MAX_MEMOIZE_SIZE = 500;

/**
 * A specialized version of `_.memoize` which clears the memoized function's
 * cache when it exceeds `MAX_MEMOIZE_SIZE`.
 *
 * @private
 * @param {Function} func The function to have its output memoized.
 * @returns {Function} Returns the new memoized function.
 */
function memoizeCapped(func) {
  var result = memoize(func, function(key) {
    if (cache.size === MAX_MEMOIZE_SIZE) {
      cache.clear();
    }
    return key;
  });

  var cache = result.cache;
  return result;
}

/** Used to match property names within property paths. */
var rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

/** Used to match backslashes in property paths. */
var reEscapeChar = /\\(\\)?/g;

/**
 * Converts `string` to a property path array.
 *
 * @private
 * @param {string} string The string to convert.
 * @returns {Array} Returns the property path array.
 */
var stringToPath = memoizeCapped(function(string) {
  var result = [];
  if (string.charCodeAt(0) === 46 /* . */) {
    result.push('');
  }
  string.replace(rePropName, function(match, number, quote, subString) {
    result.push(quote ? subString.replace(reEscapeChar, '$1') : (number || match));
  });
  return result;
});

/**
 * Converts `value` to a string. An empty string is returned for `null`
 * and `undefined` values. The sign of `-0` is preserved.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to convert.
 * @returns {string} Returns the converted string.
 * @example
 *
 * _.toString(null);
 * // => ''
 *
 * _.toString(-0);
 * // => '-0'
 *
 * _.toString([1, 2, 3]);
 * // => '1,2,3'
 */
function toString(value) {
  return value == null ? '' : baseToString(value);
}

/**
 * Casts `value` to a path array if it's not one.
 *
 * @private
 * @param {*} value The value to inspect.
 * @param {Object} [object] The object to query keys on.
 * @returns {Array} Returns the cast property path array.
 */
function castPath(value, object) {
  if (isArray(value)) {
    return value;
  }
  return isKey(value, object) ? [value] : stringToPath(toString(value));
}

/** Used as references for various `Number` constants. */
var INFINITY = 1 / 0;

/**
 * Converts `value` to a string key if it's not a string or symbol.
 *
 * @private
 * @param {*} value The value to inspect.
 * @returns {string|symbol} Returns the key.
 */
function toKey(value) {
  if (typeof value == 'string' || isSymbol(value)) {
    return value;
  }
  var result = (value + '');
  return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
}

/**
 * The base implementation of `_.get` without support for default values.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array|string} path The path of the property to get.
 * @returns {*} Returns the resolved value.
 */
function baseGet(object, path) {
  path = castPath(path, object);

  var index = 0,
      length = path.length;

  while (object != null && index < length) {
    object = object[toKey(path[index++])];
  }
  return (index && index == length) ? object : undefined;
}

/**
 * Gets the value at `path` of `object`. If the resolved value is
 * `undefined`, the `defaultValue` is returned in its place.
 *
 * @static
 * @memberOf _
 * @since 3.7.0
 * @category Object
 * @param {Object} object The object to query.
 * @param {Array|string} path The path of the property to get.
 * @param {*} [defaultValue] The value returned for `undefined` resolved values.
 * @returns {*} Returns the resolved value.
 * @example
 *
 * var object = { 'a': [{ 'b': { 'c': 3 } }] };
 *
 * _.get(object, 'a[0].b.c');
 * // => 3
 *
 * _.get(object, ['a', '0', 'b', 'c']);
 * // => 3
 *
 * _.get(object, 'a.b.c', 'default');
 * // => 'default'
 */
function get(object, path, defaultValue) {
  var result = object == null ? undefined : baseGet(object, path);
  return result === undefined ? defaultValue : result;
}

/**
 * The base implementation of `_.set`.
 *
 * @private
 * @param {Object} object The object to modify.
 * @param {Array|string} path The path of the property to set.
 * @param {*} value The value to set.
 * @param {Function} [customizer] The function to customize path creation.
 * @returns {Object} Returns `object`.
 */
function baseSet(object, path, value, customizer) {
  if (!isObject(object)) {
    return object;
  }
  path = castPath(path, object);

  var index = -1,
      length = path.length,
      lastIndex = length - 1,
      nested = object;

  while (nested != null && ++index < length) {
    var key = toKey(path[index]),
        newValue = value;

    if (key === '__proto__' || key === 'constructor' || key === 'prototype') {
      return object;
    }

    if (index != lastIndex) {
      var objValue = nested[key];
      newValue = customizer ? customizer(objValue, key, nested) : undefined;
      if (newValue === undefined) {
        newValue = isObject(objValue)
          ? objValue
          : (isIndex(path[index + 1]) ? [] : {});
      }
    }
    assignValue(nested, key, newValue);
    nested = nested[key];
  }
  return object;
}

/**
 * Sets the value at `path` of `object`. If a portion of `path` doesn't exist,
 * it's created. Arrays are created for missing index properties while objects
 * are created for all other missing properties. Use `_.setWith` to customize
 * `path` creation.
 *
 * **Note:** This method mutates `object`.
 *
 * @static
 * @memberOf _
 * @since 3.7.0
 * @category Object
 * @param {Object} object The object to modify.
 * @param {Array|string} path The path of the property to set.
 * @param {*} value The value to set.
 * @returns {Object} Returns `object`.
 * @example
 *
 * var object = { 'a': [{ 'b': { 'c': 3 } }] };
 *
 * _.set(object, 'a[0].b.c', 4);
 * console.log(object.a[0].b.c);
 * // => 4
 *
 * _.set(object, ['x', '0', 'y', 'z'], 5);
 * console.log(object.x[0].y.z);
 * // => 5
 */
function set(object, path, value) {
  return object == null ? object : baseSet(object, path, value);
}

/**
 * @desc Execute the calculation for all controls and outputs & update the value
 *
 * {this.computationAPIs} object contains the function definition of all APIs
 * The function name matches the "api" property of each output
 */
const calculateResultants = function () {
  let promiseList = [];
  const apiArgumentsObj = getApiArgumentsObj.call(this);
  [
    ...getAllTabItemsByCategory.call(this, "controls"),
    ...getAllTabItemsByCategory.call(this, "outputs"),
  ]
    .filter(isIncludesApiFunc.bind(this))
    .forEach((item) => {
    let deferred = new DeferredPromise();
    promiseList.push(deferred.promise);
    setTimeout(() => {
      const updatedValues = this.computationAPIs[item.api].call(this.computationAPIs, apiArgumentsObj);
      const itemPath = getPathById(this.config, [item.id]);
      Object.keys(updatedValues).forEach(key => {
        setPathValue.call(this, [...itemPath, key], updatedValues[key]);
      });
      deferred.resolve();
    });
  });
  return Promise.all(promiseList);
};
/**
 * To update selected tab items based on mappings
 */
const updateCurrentTabWithMappingsFromOtherTabs = function () {
  ["controls", "outputs"].forEach(category => {
    Object.values(this.config.tabs[this.selectedTabId][category] || {})
      .filter((item) => item.mappings != undefined)
      .forEach((item) => {
      const basePath = [
        "config",
        "tabs",
        this.selectedTabId,
        category,
        item.id,
      ];
      Object.keys(item.mappings).forEach(mapping => {
        const valueToBeUpdated = get(this, getPathById(this.config, item.mappings[mapping].split(".")));
        if (typeof valueToBeUpdated !== "number" ||
          !isNaN(valueToBeUpdated)) {
          setPathValue.call(this, [...basePath, ...mapping.split(".")], valueToBeUpdated);
        }
      });
    });
  });
};
/**
 * To create a deferred promise
 */
const DeferredPromise = function () {
  this.promise = new Promise((resolve, reject) => {
    this.resolve = resolve;
    this.reject = reject;
  });
};
/**
 * To get API arguments object
 * @returns API arguments object
 */
const getApiArgumentsObj = function () {
  let argumentsObj = this.parameterConfig
    ? {
      selectedDevice: this.selectedDevice,
      parameterConfigData: this.parameterConfig,
    }
    : {};
  Object.values(this.config["tabs"] || {}).forEach((tab) => {
    ["constants", "controls", "outputs"].forEach(group => {
      Object.values(tab[group] || {}).forEach((item) => {
        argumentsObj[item.id] = get(this, [
          "config",
          "tabs",
          tab.id,
          group,
          item.id,
        ]);
      });
    });
  });
  return argumentsObj;
};
/**
 * @desc get path of any files in the config folder
 * @param configName name of the configuration needed (Refer settings.ts)
 * @param deviceName selected device folder name
 * @returns file path
 */
const getConfigFilePath = function (configName, deviceName, toolName) {
  switch (configName) {
    case HOME_CONFIG:
      return `${configDirPath}${homeInfoDirName}/${deviceName}.json`;
    case JSON_CONFIG:
      return `${configFilesFolderPath}${toolName}.json`;
    case CALCULATION_CONFIG:
      if (toolName.toLowerCase() == "cm-range" &&
        (deviceName.toUpperCase() == "ADS131M04" ||
          deviceName.toUpperCase() == "ADS131M03" ||
          deviceName.toUpperCase() == "ADS131M02" ||
          deviceName.toUpperCase() == "AMC131M03" ||
          deviceName.toUpperCase() == "AMC131M02" ||
          deviceName.toUpperCase() == "AMC131M01" ||
          deviceName.toUpperCase() == "ADS131M08" ||
          deviceName.toUpperCase() == "ADS131M06" ||
          deviceName.toUpperCase() == "ADS131B04-Q1")) {
        return `${configFilesFolderPath}${toolName}-device-specific-calculation.js`;
      }
      else {
        return `${configFilesFolderPath}${toolName}-calculation.js`;
      }
    case PARAMETER_JSON_CONFIG:
      return `${configDirPath}${toolName}-parameters.json`;
    default:
      return "";
  }
};
/**
 * To get all tab items in the given category
 * @param category
 * @returns All tab items in the given category
 */
let getAllTabItemsByCategory = function (category) {
  return Object.values(this.config.tabs || {}).reduce((acc, tab) => {
    acc.push(...Object.values(tab[category] || {}));
    return acc;
  }, []);
};
/**
 * To get element key
 * @param info
 * @returns Key
 */
const getElementKey = info => {
  return info.id + (typeof info.unit === "object" ? info.unit.value : "");
};
/**
 * To get graph annotation
 * @param annotationInfo
 * @returns Graph annotation
 */
const getGraphAnnotation = (annotationInfo) => {
  const fontAttrs = {
    size: 12,
  };
  if (annotationInfo.fontColor !== undefined) {
    fontAttrs["color"] = annotationInfo.fontColor;
  }
  if (annotationInfo.fontSize !== undefined) {
    fontAttrs["size"] = annotationInfo.fontSize;
  }
  return Object.assign(Object.assign({ text: `<b>${annotationInfo.text}</b>`, showarrow: false, xref: "x", yref: "y", xanchor: annotationInfo.xanchor || "center", yanchor: annotationInfo.yanchor || "middle", x: annotationInfo.x, y: annotationInfo.y, font: Object.assign({}, fontAttrs), textangle: annotationInfo.textangle || "0" }, annotationInfo.arrowInfo), annotationInfo.otherOptions);
};
/**
 * To get graph data object
 * @param lineInfoList
 * @returns Graph data object
 */
const getGraphDataObj = lineInfoList => {
  return lineInfoList.map(lineInfo => {
    return getGraphLine(lineInfo);
  });
};
/**
 * To get graph image object
 * @param imgInfo
 * @returns Graph image object
 */
const getGraphImage = (imgInfo) => {
  return {
    source: imgInfo.source,
    xref: "x",
    yref: "y",
    x: imgInfo.x,
    y: imgInfo.y,
    sizex: imgInfo.sizex,
    sizey: imgInfo.sizey,
    xanchor: "center",
    yanchor: "middle",
  };
};
/**
 * To get graph layout object
 */
const getGraphLayoutObj = (annotationInfoList = [], imgInfoList = [], xaxis = {}, yaxis = {}, graphTitle = "", otherOptions = {}) => {
  return Object.assign({ annotations: annotationInfoList.map(annotationInfo => getGraphAnnotation(annotationInfo)), images: imgInfoList.map(imgInfo => getGraphImage(imgInfo)), margin: {
      b: 50,
      l: 70,
      pad: 10,
      r: 20,
      t: 20,
    }, showlegend: false, title: {
      text: graphTitle,
      font: {
        size: 14,
      },
    }, xaxis,
    yaxis }, otherOptions);
};
/**
 * To get graph line object
 * @param lineInfo
 * @returns Graph line object
 */
const getGraphLine = (lineInfo) => {
  const lineAttrs = {};
  // optional line attributes
  ["color", "dash", "width"].forEach(attr => {
    lineInfo[attr] != undefined && (lineAttrs[attr] = lineInfo[attr]);
  });
  return Object.assign({ x: lineInfo.x, y: lineInfo.y, name: lineInfo.name, hoverinfo: lineInfo.hoverinfo || "x+y", mode: "lines", line: Object.assign({}, lineAttrs) }, lineInfo.otherOptions);
};
/**
 * To get the json data from the given url
 * @param url
 */
const getJsonData = function (url) {
  let deferred = new DeferredPromise();
  fetch(url)
    .then(res => {
    if (res.ok) {
      return res.json();
    }
    else {
      throw new Error("Network response was not OK");
    }
  })
    .then(res => {
    console.log("url: ", url, "\ndata: ", res);
    deferred.resolve(res);
  })
    .catch(err => {
    console.log(`Unable to load/parse ${url}: ${err}`);
    deferred.reject(err);
  });
  return deferred.promise;
};
/**
 * To get value of the given control/constant/output id
 * @param id
 * @returns value of the control/constant/output
 */
const getIDValue = function (id) {
  return get(this, getPathById(this.config, [id, "value"]));
};
/**
 * To get path of the given ID
 * @param configObj
 * @param idSubPath
 * @returns Path of the given ID
 */
const getPathById = function (configObj, idSubPath) {
  let path;
  Object.keys(configObj["tabs"] || {}).some((tabId) => {
    const tab = configObj["tabs"][tabId];
    return ["constants", "controls", "outputs"].some(group => {
      return Object.keys(tab[group] || {}).some((itemId) => {
        if (itemId === idSubPath[0]) {
          path = ["config", "tabs", tabId, group, ...idSubPath];
          return true;
        }
      });
    });
  });
  if (!path) {
    console.error(`Path: ${idSubPath.join(".")} is not found in the configuration JSON`);
  }
  return path;
};
/**
 * @param str
 * @param noOfHexDigits
 * @returns Hex string with 0x prefix and given size (optional, defaults to 2)
 */
const getHexStr = function (str, noOfHexDigits = 2) {
  return `0x${String(str).toUpperCase().padStart(noOfHexDigits, "0")}`;
};
/**
 * To get selected tab/common tab
 * @returns Selected/common tab details
 */
const getTabDetails = function (tabs, selectedTabId) {
  return tabs[selectedTabId !== undefined ? selectedTabId : COMMON_TAB_ID];
};
/**
 * To get selected tab items
 * @returns Selected tab controls
 */
const getSelectedTabItems = function (category) {
  const commonTab = getTabDetails(this.config.tabs);
  const commonTabItems = Object.values((commonTab && commonTab[category]) || {}).map((info) => {
    return Object.assign(Object.assign({}, info), { tabId: commonTab.id });
  });
  const selectedTab = getTabDetails(this.config.tabs, this.selectedTabId);
  const selectedTabItems = Object.values((selectedTab && selectedTab[category]) || {}).map((info) => {
    return Object.assign(Object.assign({}, info), { tabId: selectedTab.id });
  });
  return getSortedValuesByDispPriority([
    ...commonTabItems,
    ...selectedTabItems,
  ]);
};
/**
 * To get control groups info
 * @returns Control groups info
 */
const getControlsGroupInfo = function () {
  const commonTab = getTabDetails(this.config.tabs);
  const selectedTab = getTabDetails(this.config.tabs, this.selectedTabId);
  const controlGroups = getSortedValuesByDispPriority([
    ...((commonTab && commonTab["controlGroups"]) || []),
    ...((selectedTab && selectedTab["controlGroups"]) || []),
  ]);
  let result = {
    noOfGroups: controlGroups.length,
    groupInfo: {},
  };
  controlGroups.forEach((controlGroupInfo, index) => {
    controlGroupInfo.controlIds.forEach(controlId => {
      result.groupInfo[controlId] = {
        index,
        title: controlGroupInfo.title,
        class: controlGroupInfo.class,
      };
    });
  });
  return result;
};
/**
 * To get control section
 * @returns control section
 */
const getControlSection = function () {
  const controlsGroupInfo = getControlsGroupInfo.call(this);
  let uiElements = [];
  for (let i = 0; i <= controlsGroupInfo.noOfGroups; i++) {
    uiElements.push({
      title: "",
      class: "",
      controls: [],
    });
  }
  ["controls", "constants"].forEach(category => {
    const elementsList = getSelectedTabItems.call(this, category);
    const constControlInfo = {
      controlType: "text-input",
      format: "text",
      isDisabled: true,
    };
    elementsList.forEach((itemInfo) => {
      const dataPath = [
        "config",
        "tabs",
        itemInfo.tabId,
        category,
        itemInfo.id,
        "value",
      ];
      const itemGroupDetails = controlsGroupInfo.groupInfo[itemInfo.id];
      let groupIndex = 0;
      if (category === "constants") {
        itemInfo = Object.assign(Object.assign({}, itemInfo), constControlInfo);
      }
      if (itemGroupDetails) {
        groupIndex = itemGroupDetails.index + 1;
        if (uiElements[groupIndex].title === "") {
          uiElements[groupIndex].title = itemGroupDetails.title;
          uiElements[groupIndex].class = itemGroupDetails.class;
        }
      }
      uiElements[groupIndex].controls.push(this.getControlElement(itemInfo, dataPath));
    });
  });
  return uiElements;
};
/**
 * To get output section
 * @returns output section
 */
const getOutputSection = function () {
  const elementsList = getSelectedTabItems.call(this, "outputs");
  let uiElements = [];
  elementsList
    .filter(itemInfo => {
    var _a;
    return this.selectedOutputSubCategoryId != undefined
      ? (_a = itemInfo.subCategory) === null || _a === void 0 ? void 0 : _a.includes(this.selectedOutputSubCategoryId)
      : true;
  })
    .forEach((itemInfo) => {
    const dataPath = [
      "config",
      "tabs",
      itemInfo.tabId,
      "outputs",
      itemInfo.id,
      "value",
    ];
    uiElements.push(this.getOutputElement(itemInfo, dataPath));
  });
  return uiElements;
};
/**
 * To get sort by display priority
 * @param items
 * @returns Sorted items list
 */
const getSortedValuesByDispPriority = function (items) {
  return items.sort(sortByDisplayPriorityDesc);
};
/**
 * To get tab names list
 * @returns Tab names list
 */
const getTabNamesObj = function (tabs) {
  return Object.values(tabs)
    .filter(tab => tab.id !== COMMON_TAB_ID)
    .reduce((tabNamesObj, tab) => {
    tabNamesObj[tab.id] = tab.label;
    return tabNamesObj;
  }, {});
};
/**
 * To get subcategory names as an Object
 * @returns Subcategory names object
 */
const getOutputSubCategoriesObj = function () {
  let subCategoryNamesObj = {};
  getSelectedTabItems.call(this, "outputs").forEach(item => {
    var _a;
    ((_a = item.subCategory) === null || _a === void 0 ? void 0 : _a.length) &&
      item.subCategory.forEach(subCategory => {
        subCategoryNamesObj[subCategory] = subCategory;
      });
  });
  return subCategoryNamesObj;
};
/**
 * To check whether the item (control, constant, output) includes an API function
 * @param item
 * @returns True, if item includes an API function
 */
const isIncludesApiFunc = function (item) {
  return typeof this.computationAPIs[item.api] === "function";
};
/**
 * To update the configuration object with the JSON data
 * @param configObjName Name of the config state variable
 * @param jsonUrl JSON url
 * @returns Promise
 */
const loadAndUpdateConfigObj = function (configObjName, jsonUrl) {
  let deferred = new DeferredPromise();
  getJsonData(jsonUrl)
    .then(data => {
    this[configObjName] = data;
    deferred.resolve(true);
  })
    .catch(err => {
    deferred.reject(err);
  });
  return deferred.promise;
};
/**
 * To load and update page configuration for tools
 * @param jsonConfig
 * @param calculationConfig
 */
const loadAndUpdatePageConfig = async function (toolName, deviceName) {
  const configJsonPath = getConfigFilePath(JSON_CONFIG, "", toolName);
  const configCalcPath = getConfigFilePath(CALCULATION_CONFIG, deviceName, toolName);
  let apiModule = {};
  // 1 - get the computation APIs needed for the device
  try {
    apiModule = await import(`../../${configCalcPath}`);
  }
  catch (e) {
    console.log(e);
  }
  finally {
    this.computationAPIs = Object.assign({}, apiModule);
  }
  // 2 - load the base config
  await loadAndUpdateConfigObj.call(this, `config`, configJsonPath);
  // 3 - load the parameter config
  try {
    switch (toolName) {
      case "crc":
      case "crc-s0x":
      case "digital-filter":
      case "tue":
      case "cm-range":
      case "code-conversion":
        let validToolName = toolName.startsWith("crc") ? "crc" : toolName;
        const parameterConfigJsonPath = getConfigFilePath(PARAMETER_JSON_CONFIG, "", validToolName);
        await loadAndUpdateConfigObj.call(this, `parameterConfig`, parameterConfigJsonPath);
        validateFileData(parameterConfigJsonPath, this.parameterConfig, this.selectedDevice);
        updateControlObject(this.config, this.parameterConfig[this.selectedDevice]);
        break;
    }
  }
  catch (error) {
    console.log("%c" + error, "color:red;font-size:18px;");
  }
  try {
    if (toolName === "digital-filter") {
      let lutFileURL = this.parameterConfig[this.selectedDevice].lutFileURL;
      const pattern = /^(https?:\/\/)?([\w.-]+)\.([a-zA-Z]{2,})(\/\S*)?$/;
      if (pattern.test(lutFileURL)) {
        var xmlHttp = new XMLHttpRequest();
        xmlHttp.open("GET", lutFileURL, true); // false for synchronous request
        xmlHttp.onload = e => {
          if (xmlHttp.readyState === 4 && xmlHttp.status === 200) {
            let jsonData = JSON.parse(xmlHttp.responseText);
            this.parameterConfig[this.selectedDevice].lutData = jsonData[0]
              ? jsonData[0]
              : jsonData;
            this.parameterConfigUpdated = true;
            forceUpdate(this.host);
          }
          else if (xmlHttp.statusText === "Not Found" ||
            xmlHttp.status === 404) {
            this.errorMessage = `LUT File for ${this.selectedDevice} in not present`;
            this.parameterConfigUpdated = true;
          }
          else if (xmlHttp.statusText === "error" ||
            xmlHttp.statusText === "NetworkError") {
            this.errorMessage = `CORS error occurred for the LUT File of ${this.selectedDevice}`;
            this.parameterConfigUpdated = true;
          }
          else {
            this.errorMessage = `Other error: ${xmlHttp.statusText} for the LUT File of ${this.selectedDevice}`;
            this.parameterConfigUpdated = true;
          }
        };
        xmlHttp.send();
        let digitalFilterCalculationFilePath = `../../${digitalFilterCalculationFilesFolderPath}${this.selectedDevice.toLowerCase()}_digital-filter-calculation.js`;
        this.parameterConfig[this.selectedDevice].calculationModule =
          await import(digitalFilterCalculationFilePath);
        if (Object.keys(this.parameterConfig[this.selectedDevice].calculationModule).length > 0) {
          validateFileData(digitalFilterCalculationFilePath, this.parameterConfig, this.selectedDevice);
        }
      }
      else {
        this.errorMessage =
          lutFileURL === undefined
            ? `LUT File URL parameter for ${this.selectedDevice} is missing`
            : `Invalid LUT File URL for ${this.selectedDevice}`;
        this.parameterConfigUpdated = true;
      }
    }
    else if (toolName === "code-conversion") {
      let codeConversionCalculationFilePath = `../../${codeConversionCalculationFilesFolderPath}${this.selectedDevice.toLowerCase()}_code-conversion-calculation.js`;
      this.parameterConfig[this.selectedDevice].calculationModule =
        await import(codeConversionCalculationFilePath);
    }
    else if (toolName === "tue") {
      this.isInitializationApisDeleted = false;
    }
  }
  catch (error) {
    console.log("%c" + error, "color:red;font-size:18px;");
  }
  let configObjName = "config";
  let configObj = Object.assign({}, this[configObjName]);
  // 3 - update ID's
  configObj = updateIdValues(configObj);
  // trigger the config state object change
  this[configObjName] = null;
  this[configObjName] = Object.assign({}, configObj);
  // 3 - update resultants
  if (toolName !== "digital-filter" || this.parameterConfigUpdated) {
    updateResultants.call(this, true);
  }
};
/**
 * To set the datapath value
 * @param dataPath
 * @param value
 */
const setPathValue = function (dataPath, value) {
  set(this, dataPath, value);
};
/**
 * To sort by display priority in descending order
 * @param a
 * @param b
 * @returns Difference between display priority of a and b
 */
const sortByDisplayPriorityDesc = function (a, b) {
  a.displayPriority = a.displayPriority || 0;
  b.displayPriority = b.displayPriority || 0;
  return b.displayPriority - a.displayPriority;
};
/**
 * Set path value event handler
 */
const updateConfigData = async function (ev, canUpdateResultant = true) {
  setPathValue.call(this, ev.detail.dataPath, ev.detail.value);
  if (canUpdateResultant) {
    await updateResultants.call(this, true);
  }
};
/**
 * @desc Update ID values
 * @param configObj config object
 * @returns {object} updated config object
 */
const updateIdValues = function (configObj) {
  Object.keys(configObj.tabs).forEach(tabId => {
    const tabPath = ["tabs", tabId];
    set(configObj, [...tabPath, "id"], tabId);
    ["controls", "constants", "outputs"].forEach(category => {
      const categoryPath = [...tabPath, category];
      Object.keys(get(configObj, categoryPath) || {}).forEach(itemId => {
        const itemPath = [...categoryPath, itemId];
        set(configObj, [...itemPath, "id"], itemId);
      });
    });
  });
  return configObj;
};
/**
 * @desc execute all calculations
 * @param needForceUpdate component will be forced to render when passed true
 */
const updateResultants = async function (needForceUpdate) {
  if (this.config != undefined) {
    await calculateResultants.call(this).then(() => {
      if (!this.isComputedValuesInitialized) {
        this.isComputedValuesInitialized = true;
      }
      if (needForceUpdate) {
        forceUpdate(this.host);
      }
    });
  }
};
/**
 * Update selected output sub category
 */
const updateSelectedOutputSubCategoryId = function (ev) {
  ev.stopPropagation();
  this.selectedOutputSubCategoryId = ev.detail.id;
};
/**
 * Update currently selected category tab
 */
const updateSelectedTabId = function (ev) {
  ev.stopPropagation();
  this.selectedTabId = ev.detail.id;
};
const updateControlObject = function (configObj, parameterJSONData) {
  if (parameterJSONData && parameterJSONData.controls) {
    for (let tab in parameterJSONData.controls) {
      if (parameterJSONData.controls.hasOwnProperty(tab)) {
        Object.assign(configObj.tabs[tab].controls, parameterJSONData.controls[tab]);
      }
    }
  }
};
// ---------- Popup helper functions ----------
/**
 * To close popup
 */
const closePopup = function () {
  this.isShowPopup = false;
};
/**
 * To trigger close popup call when user clicks outside
 * @param ev
 */
const closePopupListener = function (ev) {
  if (isUserClickedOutsidePopup.call(this, ev)) {
    closePopup.call(this);
    window.removeEventListener("click", this.closePopupListener);
  }
};
/**
 * To check whether the user clicked outside the popup
 * @param ev Pointer event
 * @returns Boolean
 */
const isUserClickedOutsidePopup = function (ev) {
  return !(this.popupDiv.shadowRoot
    .querySelector(".popup .header")
    .contains(ev.composedPath()[0]) ||
    this.popupContentDiv.contains(ev.composedPath()[0]));
};
/**
 * To show popup
 */
const showPopup = function (ev) {
  ev.stopPropagation();
  this.closePopupListener = closePopupListener.bind(this);
  window.addEventListener("click", this.closePopupListener);
  this.isShowPopup = true;
};

export { getGraphLayoutObj as a, getGraphDataObj as b, updateResultants as c, getOutputSubCategoriesObj as d, updateSelectedOutputSubCategoryId as e, getOutputSection as f, getControlSection as g, getTabNamesObj as h, updateSelectedTabId as i, getElementKey as j, updateConfigData as k, loadAndUpdatePageConfig as l, isSymbol as m, getIDValue as n, loadAndUpdateConfigObj as o, getConfigFilePath as p, getHexStr as q, showPopup as s, updateCurrentTabWithMappingsFromOtherTabs as u };

//# sourceMappingURL=p-f53384b7.js.map